
/*
 *  $Id: regen.h,v 1.3 2000/09/02 22:38:18 bruce Exp $
 */

#ifndef REGEN_H
#define REGEN_H

#include <sys/types.h>

#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <sys/fcntl.h>
#include <sys/mman.h>
#include <sys/param.h>
#include <sys/stat.h>

#ifndef PATH_MAX
#  define PATH_MAX 4096
#endif

#define tSCC static const char
#define tCC  const char
typedef unsigned long ulong_t;

typedef enum { B_FALSE = 0, B_TRUE } t_bool;
typedef int    t_success;

#define PROBLEM  ((t_success)1)
#define SUCCESS  ((t_success)0)
#define FAILURE  ((t_success)-1)

#define SUCCEEDED(r)  ((r)==SUCCESS)
#define FAILED(r)     ((r)<SUCCESS)
#define HAD_GLITCH(r) ((r)>SUCCESS)
#define SUCCESSFUL(r) SUCCEEDED(r)


#ifdef DEFINING
#  define VALUE(v)  = v
#  define EXPORT
#else
#  define VALUE(v)
#  define EXPORT extern
#endif

EXPORT char* pz_work_dir  VALUE( "/tmp/regen" );
EXPORT char* pz_queue     VALUE( NULL );
EXPORT char* pz_failure   VALUE( NULL );

EXPORT char* pz_cvsroot   VALUE( NULL );  /* AKA  $CVSROOT  */
EXPORT char* pz_project   VALUE( NULL );  /* directory under $CVSROOT */

/*
 *  CVS layout can sometimes be a bit weird.  "pz_myroot" is wherever
 *  the user happened to decide to build the hierarchy.  "pz_myproj"
 *  is the subdirectory under that where the currently committed files
 *  are coming from.  These are determined by counting the number of
 *  subdirectories in "pz_project" and backing off that many directory
 *  entries for the "pz_myproj" and the remainder are "pz_myroot".  Ugly.
 */
EXPORT char* pz_myroot    VALUE( NULL );
EXPORT char* pz_myproj    VALUE( NULL );

/*
 *  Memory map of the CVS/Entries file
 */
EXPORT tCC*  pz_entries   VALUE( NULL );

/*
 *  CVS Version tag ("-N" for top-of-tree)
 */
EXPORT const char* pz_ver_tag    VALUE( NULL );

/*
 *  If it becomes necessary to fork so that our "parent" commit may
 *  proceed, indicate that with this so we don't do it more than once.
 */
EXPORT t_bool      have_forked   VALUE( B_FALSE );



#define xfree( p )  free((void*)p)

#ifndef DEFINING

void*   xmalloc(  size_t size );
void*   xrealloc( void* p, size_t size );
char*   xstrdup( tCC* );

#else

tSCC zAlocFail[] = "allocation failed for %d bytes\n";


void* xmalloc( size_t size )
{
    void* res = malloc( size );
    if (res == NULL) {
        fprintf( stderr, zAlocFail, size );
        exit( EXIT_FAILURE );
    }
    return res;
}

char* xstrdup( tCC* pz )
{
    size_t len = strlen( pz ) + 1;
    char*  res = (char*)xmalloc( len );
    strcpy( res, pz );
    return res;
}

void* xrealloc( void* p, size_t size )
{
    void* res = (p==NULL) ? malloc( size ) : realloc( p, size );
    if (res == NULL) {
        fprintf( stderr, zAlocFail, size );
        exit( EXIT_FAILURE );
    }
    return res;
}
#endif /* DEFINING */

#endif /* REGEN_H */
